package massim.demoAgents;

import java.util.Random;
import java.util.Vector;

import massim.cowmonitor.*;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Description of an agent for the simulation
 * 
 * @author Slawomir Deren
 * 
 */
public class DemoGridAgent1 extends AbstractAgent {

	private int gsizeX = 0, gsizeY = 0;
	private int posX = 0, posY = 0;
	private Vector<String> vectorC = null;
	private Vector<String> vectorD = null;

	private int homex1;
	private int homex2;
	private int homey1;
	private int homey2;

	private int lineOfSight = 0;

	private int state = 1;

	private scoutbehaviour sb;
	private surroundbehaviour sub = new surroundbehaviour();
	private pushbehaviour pb;

	/**
	 * Main method of this class
	 * 
	 * @param args
	 *            Username, Passwort and optional Host
	 */
	public static void main(String[] args) {

		// basic configuration settings
		/*
		
		*/
		DemoGridAgent1 demoagent = new DemoGridAgent1();
		 
		demoagent.setPort(12300);

		if (args.length == 3) {
			demoagent.setHost(args[2]);
		}

		else {
			 demoagent.setHost("localhost");
		}

		demoagent.setUsername(args[0]);
		demoagent.setPassword(args[1]);
		demoagent.start();
		
	}
	public DemoGridAgent1(){}
public DemoGridAgent1(String host,String username, String pass){
	this.setUsername(username);
	this.setPassword(pass);
	this.setHost(host);
	this.setPort(12300);
}
	/*
	 * (non-Javadoc)
	 * 
	 * @see massim.AbstractAgent#processRequestAction(org.w3c.dom.Element,
	 *      org.w3c.dom.Element, long, long)
	 */
	@SuppressWarnings("static-access")
	public void processRequestAction(Element perception, Element target,
			long currenttime, long deadline) {

		// react to normal perception, modify target so that it becomes a valid
		// action
		String action = "";
		String[][] a;
		a = processPerception(perception);

		posX = Integer.parseInt(perception.getAttribute("posx"));
		posY = Integer.parseInt(perception.getAttribute("posy"));
		actionStateClass ActionState = new actionStateClass();
		if (state == 0) {
			ActionState = pb.action(a, posX, posY);

		}
		if (state == 1) {

			ActionState = sb.action(state, a, posX, posY);

		}

		if (state == 2) {
			ActionState = sub.action();

		}

		action = ActionState.action;
		state = ActionState.state;
		

		// set the action
		target.setAttribute("type", action);
		vectorC.clear();
		vectorD.clear();
	}

	public String[][] processPerception(Element a) {
		int i;
		String b[][] = new String[lineOfSight][lineOfSight];

		NodeList nl = a.getElementsByTagName("cell");
		for (i = 0; i < nl.getLength(); i++) {
			Node n = nl.item(i);
			Element ne = (Element) n;
			NodeList nnl = n.getChildNodes();

			String content = nnl.item(1).getNodeName();
			int directionX = Integer.parseInt(ne.getAttribute("x"));
			int directionY = Integer.parseInt(ne.getAttribute("y"));

			b[directionX + lineOfSight / 2][directionY + lineOfSight / 2] = content;

		}

		return b;
	}

	/**
	 * Determination of random action
	 * 
	 * @return new action Random action
	 */
	public String randomAction() {

		Random r1 = new Random();
		String action = "skip";

		// action generating by random number
		switch (r1.nextInt(2)) {
		case 0:
			action = "right";
			break;
		case 1:
			action = "left";
			break;
		}

		// return action
		return action;
	}

	public String surround() {
		return "down";
	}

	public String push() {
		return "left";
	}

	/**
	 * Computation of the better direction (shortest way) to the depot
	 * 
	 * @param up
	 *            Distance to the depot (from the adjacent up cell)
	 * @param left
	 *            Distance to the depot (from the adjacent left cell)
	 * @param down
	 *            Distance to the depot (from the adjacent down cell)
	 * @param right
	 *            Distance to the depot (from the adjacent right cell)
	 * @return The better direction
	 */
	public String determineDirection(int up, int left, int down, int right) {

		String direction = "skip";

		// "up"
		if (up <= left && up <= down && up <= right) {

			direction = "up";
		}

		// "left"
		else if (left <= up && left <= down && left <= right) {

			direction = "left";
		}

		// "down"
		else if (down <= up && down <= left && down <= right) {

			direction = "down";
		}

		// "right"
		else if (right <= up && right <= left && right <= down) {

			direction = "right";
		}

		// return action
		return direction;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see massim.AbstractAgent#processLogIn()
	 */
	public void processLogIn() {
		// called as soon as agent logged in successfully
		// TODO: insert code here
		// Note: This is a good place to do a ping
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see massim.AbstractAgent#processPong(java.lang.String)
	 */
	public void processPong(String pong) {
		// react on incoming pong
		// TODO: insert code here
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see massim.AbstractAgent#processSimulationEnd(org.w3c.dom.Element, long,
	 *      long)
	 */
	public void processSimulationEnd(Element perception, long currenttime) {
		// react on simulation end
		// TODO: insert code here
	}

	/**
	 * Initialisation of main variables and vectors
	 */
	public void processSimulationStart(Element perception, long currenttime) {

		// react on simulation Start
		gsizeX = Integer.parseInt(perception.getAttribute("gsizex"));
		gsizeY = Integer.parseInt(perception.getAttribute("gsizey"));

		lineOfSight = 17;

		homex1 = Integer.parseInt(perception.getAttribute("corralx0"));
		homex2 = Integer.parseInt(perception.getAttribute("corralx1"));
		homey1 = Integer.parseInt(perception.getAttribute("corraly0"));
		homey2 = Integer.parseInt(perception.getAttribute("corraly1"));

		// lineOfSight = 5;
		// //Integer.parseInt(perception.getAttribute("lineOfSight"));

		sb = new scoutbehaviour(gsizeX, gsizeY, posX, posY);
		pb = new pushbehaviour(gsizeX, gsizeY, homex1, homey1, homex2, homey2);

		vectorC = new Vector<String>();
		vectorD = new Vector<String>();
	}
}